using System;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.DependencyInjection;
using PaymentService;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Domain.OrderRecordAggregate;
using PXPlus.Payment.Infrastructure;
using Xunit;

namespace PXPlus.Payment.IntegrationTest
{
    public class RefundIntegrationTest : IClassFixture<TestServerFixture>
    {
        
        private readonly PaymentGrpcService.PaymentGrpcServiceClient _paymentGrpcService;
        private readonly RefundGrpcService.RefundGrpcServiceClient _refundGrpcServiceClient;
        private readonly IServiceProvider _rootServices;
        
        public RefundIntegrationTest(TestServerFixture testServerFixture)
        {
            _paymentGrpcService = new PaymentGrpcService.PaymentGrpcServiceClient(testServerFixture.GrpcChannel);
            _refundGrpcServiceClient = new RefundGrpcService.RefundGrpcServiceClient(testServerFixture.GrpcChannel);
            _rootServices = testServerFixture.Services;
        }

        private PaymentGrpcRequest PaymentReq(double amount,double noneFeedbackAmount,double noneDiscountAmount)
            => new()
            {
                TradeNo = Guid.NewGuid().ToString(),
                PayerId = 1,
                PayToolType = 3,
                PayToolSeq = 1,
                IsUsePoint = true,
                StoreCode = "store_code",
                Amount = amount,
                NoneFeedbackAmount = noneFeedbackAmount,
                NoneDiscountAmount = noneDiscountAmount,
                PlatformType = 2,
                IsEcTrade = false,
                MerchantTradeNo = "a",
                MemberPhoneNo = "a"
            };
        

        private RefundGrpcRequest RefundReq(string oriPxTradeNo,double amount, double noneDiscountAmount, double noneFeedbackAmount)
            => new()
            {
                TradeNo = Guid.NewGuid().ToString(),
                OriPxTradeNo = oriPxTradeNo,
                Amount = amount,
                NoneDiscountAmount = noneDiscountAmount,
                NoneFeedbackAmount = noneFeedbackAmount,
                RefunderId = 1,
                StoreCode = "store_code",
                MerchantTradeNo = "as"
            };
        
        /// <summary>
        ///  arrange 付款amount = 1000 , 不可折抵 = 300 ,點數折抵 (1000-300)*30%=210 ,payAmount = 790
        ///  action 退款 200 , 不可折抵 = 100 ,點數折抵((1000-200)-(300-100))*30%=180, payAmount= 620
        ///  assert amount = 200 , payAmount =  170 return 點數 = 30點
        /// </summary>
        [Fact]
        public async Task Case1()
        {
            //arrange
            var paymentResponse = _paymentGrpcService.PaymentCommand(PaymentReq(1000,20,300));
            
            
            Assert.Equal("0000",paymentResponse.Code);
            
            Assert.Equal(1000,paymentResponse.Data.OrderAmount);
            Assert.Equal(790,paymentResponse.Data.PayAmount);
            Assert.Equal(210,paymentResponse.Data.DiscountPoint);
            Assert.Equal(300,paymentResponse.Data.NoneDiscountAmount);
            Assert.Equal(20,paymentResponse.Data.NoneFeedbackAmount);
            
            //action

            var refundResponse =
                _refundGrpcServiceClient.RefundCommand(RefundReq(paymentResponse.Data.PxTradeNo, 200, 100, 5));
            
            
            // assert
            Assert.Equal("0000",refundResponse.Code);
            
            Assert.Equal(200,refundResponse.Data.Amount);
            Assert.Equal(170,refundResponse.Data.TradeAmount);
            Assert.Equal(30,refundResponse.Data.DiscountPoint);

            
            using var serviceScope = _rootServices.CreateScope();
            var orderRepository = serviceScope.ServiceProvider.GetRequiredService<IOrderRepository>();
            var order = await orderRepository.GetOrderByAsync(paymentResponse.Data.PxTradeNo);
                
            Assert.Equal(800,order.OrderAmount);
            Assert.Equal(620,order.PayAmount);
            Assert.Equal(180,order.PointDiscountInfo.ActualDiscountAmount);
            Assert.Equal(200,order.NoneDiscountAmount);
            Assert.Equal(15,order.NoneFeedbackAmount);

            var paymentContext = serviceScope.ServiceProvider.GetRequiredService<PaymentContext>();

            var orderRefundRecord = await paymentContext.OrderRecords.AsNoTracking().FirstOrDefaultAsync(c => c.OrderId == order.Id && c.OrderRecordType == OrderRecordType.Refund);
            
            Assert.Equal(200,orderRefundRecord.OrderAmount);
            Assert.Equal(170,orderRefundRecord.PayAmount);
            Assert.Equal(30,orderRefundRecord.ActualDiscountAmount);
            Assert.Equal(100,orderRefundRecord.NoneDiscountAmount);
            Assert.Equal(5,orderRefundRecord.NoneFeedbackAmount);
        }
    }
}