using System;
using System.Threading;
using System.Threading.Tasks;
using AutoMapper;
using MediatR;
using Microsoft.Extensions.Logging;
using PXPlus.Core.Extensions;
using PXPlus.Payment.Application.Dto;
using PXPlus.Payment.Domain.DomainExceptions;

namespace PXPlus.Payment.Application.Behaviors
{
    public class ExceptionHandleBehavior<TRequest, TResponse> : IPipelineBehavior<TRequest, TResponse> where TResponse : Response
    {
        private readonly IMapper _mapper;
        private readonly ILogger<ExceptionHandleBehavior<TRequest, TResponse>> _logger;

        public ExceptionHandleBehavior(IMapper mapper, ILogger<ExceptionHandleBehavior<TRequest, TResponse>> logger)
        {
            _mapper = mapper;
            _logger = logger;
        }

        public async Task<TResponse> Handle(TRequest request, CancellationToken cancellationToken,
            RequestHandlerDelegate<TResponse> next)
        {
            try
            {
                return await next();
            }
            catch (OrderDomainException de)
            {
                var response = new Response {Code = de.ResultCode, Message = de.ResultMessage + de.AdditionalMessage};
                return _mapper.Map<TResponse>(response);
            }
            catch (Exception e)
            {
                var response = new Response
                               {
                                   Code = ((int) PaymentStatusCode.OtherError).ToString(),
                                   Message = PaymentStatusCode.OtherError.GetResulteMessage()
                               };
                _logger.LogError(e, e.Message);
                return _mapper.Map<TResponse>(response);
            }
        }
    }
}