using System;
using System.Threading;
using System.Threading.Tasks;
using MediatR;
using Microsoft.Extensions.Logging;
using PXPlus.Core.IntegrationEventLogEf.Services;
using PXPlus.Payment.Application.Commands;
using PXPlus.Payment.Infrastructure;

namespace PXPlus.Payment.Application.Behaviors
{
    public class TransactionBehavior<TRequest, TResponse> : IPipelineBehavior<TRequest, TResponse>
    {
        private readonly ILogger<TransactionBehavior<TRequest, TResponse>> _logger;
        private readonly PaymentContext _dbContext;
        private readonly IDelayPublishEventService _delayPublishEventService;

        public TransactionBehavior(ILogger<TransactionBehavior<TRequest, TResponse>> logger, PaymentContext dbContext, IDelayPublishEventService delayPublishEventService)
        {
            _logger = logger;
            _dbContext = dbContext;
            _delayPublishEventService = delayPublishEventService;
        }

        public async Task<TResponse> Handle(TRequest request, CancellationToken cancellationToken,
            RequestHandlerDelegate<TResponse> next)
        {
            //Skip IdentifiedCommand because he must has db transaction by himself.
            if (typeof(TRequest).IsGenericType && typeof(TRequest).GetGenericTypeDefinition() ==
                typeof(IdentifiedCommand<,>).GetGenericTypeDefinition())
                return await next();

            var response = default(TResponse);
            var typeName = request.GetType().Name;

            try
            {
                if (_dbContext.HasActiveTransaction)
                    return await next();

                await using var transaction = await _dbContext.BeginTransactionAsync();

                _logger.LogInformation("----- Begin transaction {TransactionId} for {CommandName} ({@Command})",
                    transaction.TransactionId, typeName, request);

                response = await next();

                _logger.LogInformation("----- Commit transaction {TransactionId} for {CommandName}",
                    transaction.TransactionId, typeName);

                await _dbContext.CommitTransactionAsync(transaction);

                var transactionId = transaction.TransactionId;

                _delayPublishEventService.AddEventToDelayPublish(transactionId);

                return response;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "ERROR Handling transaction for {CommandName} ({@Command})", typeName, request);
                //todo publish payment failed or refund failed event
                throw;
            }
        }
    }
}