using System.Collections.Generic;
using System.Linq;
using System.Text.Json;
using System.Threading;
using System.Threading.Tasks;
using FluentValidation;
using MediatR;
using Microsoft.Extensions.Logging;
using PXPlus.Payment.Domain.DomainExceptions;

namespace PXPlus.Payment.Application.Behaviors
{
    public class ValidatorBehavior<TRequest, TResponse> : IPipelineBehavior<TRequest, TResponse>
    {
        private readonly ILogger<ValidatorBehavior<TRequest, TResponse>> _logger;
        private readonly IEnumerable<IValidator<TRequest>> _validators;

        public ValidatorBehavior(ILogger<ValidatorBehavior<TRequest, TResponse>> logger,
            IEnumerable<IValidator<TRequest>> validators)
        {
            _logger = logger;
            _validators = validators;
        }

        public async Task<TResponse> Handle(TRequest request, CancellationToken cancellationToken,
            RequestHandlerDelegate<TResponse> next)
        {
            
          
            var typeName = typeof(TRequest).Name;

            _logger.LogInformation("----- Validating command {CommandType}", typeName);

            var failures = _validators
                           .Select(v => v.Validate(request))
                           .SelectMany(result => result.Errors)
                           .Where(error => error != null)
                           .ToList();

            
            if (!failures.Any())
                
                return await next();
            
            _logger.LogWarning(
                "Validation errors - {CommandType} - Command: {@Command} - Errors: {@ValidationErrors}", typeName,
                request, failures);

            throw new OrderDomainException(
                PaymentStatusCode.InvalidParameter,
                $"Command Validation Errors for type {typeof(TRequest).Name} " +
                $"Paras:{JsonSerializer.Serialize(failures.Select(c => c.PropertyName))} " +
                $"ErrorMessage:{JsonSerializer.Serialize(failures.Select(c => c.ErrorMessage))}");
            
        }
    }
}