﻿using System;
using System.Text.Json;
using System.Threading;
using System.Threading.Tasks;
using MediatR;
using PXPlus.Payment.Application.Dto;
using PXPlus.Payment.Domain.DomainExceptions;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Infrastructure.Idempotency;

namespace PXPlus.Payment.Application.Commands
{
    public class CancelPaymentWithPxTradeNoCommandHandler : IRequestHandler<CancelPaymentWithPxTradeNoCommand,Response>
    {
        private readonly IOrderRepository _orderRepository;
        private readonly IMediator _mediator;

        public CancelPaymentWithPxTradeNoCommandHandler(IOrderRepository orderRepository, IMediator mediator)
        {
            _orderRepository = orderRepository;
            _mediator = mediator;
        }
        
        public async Task<Response> Handle(CancelPaymentWithPxTradeNoCommand request, CancellationToken cancellationToken)
        {
            var order = await _orderRepository.GetOrderByNoTrackingAsync(request.PxTradeNo);

            if (order == null)
                throw new OrderDomainException(PaymentStatusCode.RefundFailedOrderNotFound);

            if (order.PayerInfo.PayerId != request.PayerId)
                throw new OrderDomainException(PaymentStatusCode.RefundFailedOrderNotFound);

            if (order.ReceiveStoreInfo.StoreCode != request.StoreCode)
                throw new OrderDomainException(PaymentStatusCode.RefundFailedOrderNotFound);

            if (order.OrderAmount <= 0)
                throw new OrderDomainException(PaymentStatusCode.RefundFailedAvailableAmountNotEnough);
            
            //只能全額退
            if (order.OrderStatus == OrderStatus.Refunded)
                throw new OrderDomainException(PaymentStatusCode.RefundFailedAvailableAmountNotEnough);
            
            if (order.OrderStatus != OrderStatus.PaymentSuccess)
                throw new OrderDomainException(PaymentStatusCode.RefundFailed,"原單非付款成功，不可退款。");
            
            var refundCommand = new RefundCommand(
                request.PayerId,
                request.StoreCode,
                request.PxTradeNo,
                order.OrderAmount,
                order.NoneFeedbackAmount,
                order.NoneDiscountAmount,
                order.ActivityCashDiscountInfos,
                DateTime.Now.ToString("CPPyyyyMMddHHmmssffff"),
                request.RefundCause,
                request.ExecuteSource,
                request.Executor);

            var refundAppResponse = await _mediator.Send(refundCommand, cancellationToken);

            return MapFrom(refundAppResponse);
        }

        private Response MapFrom(RefundAppResponse refundAppResponse)
            => new()
            {
                Code = refundAppResponse.Code,
                Message = refundAppResponse.Message,
                Prefix = refundAppResponse.Prefix
            };
    }
    
    public class CancelPaymentWithPxTradeNoIdentifierCommandHandler : IdentifiedCommandHandler<CancelPaymentWithPxTradeNoCommand, Response>
    {
        private readonly IRequestManager _requestManager;

        public CancelPaymentWithPxTradeNoIdentifierCommandHandler(IMediator mediator, IRequestManager requestManager) : base(mediator,
            requestManager)
        {
            _requestManager = requestManager;
        }

        protected override async Task<Response> CreateResultForDuplicateRequestAsync(
            IdentifiedCommand<CancelPaymentWithPxTradeNoCommand, Response> request)
        {
            var query = await _requestManager.GetAsync(request.TradeNo);

            if (query.ClientRequestType != ClientRequestType.CancelPaymentWithPxTradeNoCommand)
                throw new OrderDomainException(PaymentStatusCode.TradeNoHasBeenUsed);

            if (query.ResponseData == null)
                throw new OrderDomainException(PaymentStatusCode.TradeProcessing);

            return JsonSerializer.Deserialize<Response>(query.ResponseData);
        }
    }
}