using System;
using System.Text.Json;
using System.Threading;
using System.Threading.Tasks;
using FluentValidation;
using MediatR;
using Microsoft.Extensions.Logging;
using PXPlus.Payment.Domain.DomainExceptions;
using PXPlus.Payment.Infrastructure.Idempotency;

namespace PXPlus.Payment.Application.Commands
{
    public class IdentifiedCommandHandler<T,R> : IRequestHandler<IdentifiedCommand<T, R>, R>
        where T : IRequest<R>
    {
        private readonly IMediator _mediator;
        private readonly IRequestManager _requestManager;

        public IdentifiedCommandHandler(
            IMediator mediator,
            IRequestManager requestManager)
        {
            _mediator = mediator;
            _requestManager = requestManager;
        }
        
        /// <summary>
        /// Creates the result value to return if a previous request was found
        /// </summary>
        /// <returns></returns>
        protected virtual Task<R> CreateResultForDuplicateRequestAsync(IdentifiedCommand<T, R> request)
        {
            return default;
        }
        
        /// <summary>
        /// This method handles the command. It just ensures that no other request exists with the same ID, and if this is the case
        /// just enqueues the original inner command.
        /// </summary>
        /// <param name="request">IdentifiedCommand which contains both original command & request ID</param>
        /// <returns>Return value of inner command or default value if request same ID was found</returns>
        public async Task<R> Handle(IdentifiedCommand<T, R> request, CancellationToken cancellationToken)
        {
            var command = request.Command;

            var clientRequestType = command switch
            {
                PaymentCommand => ClientRequestType.PaymentCommand,
                RefundCommand => ClientRequestType.RefundCommand,
                OrderConfirmCommand => ClientRequestType.OrderConfirmCommand,
                CancelPaymentWithPxTradeNoCommand => ClientRequestType.CancelPaymentWithPxTradeNoCommand,
                _ => ClientRequestType.UnknownCommand
            };

            var alreadyExists = await _requestManager.ExistAsync(request.TradeNo);
            
            if (alreadyExists)
                return await CreateResultForDuplicateRequestAsync(request);

            var re = await _requestManager.CreateRequestAsync(request.TradeNo, clientRequestType);
            
            try
            {
                // Send the embedded business command to mediator so it runs its related CommandHandler 
                var result = await _mediator.Send(command, cancellationToken);

                await _requestManager.UpdateRequestOrderRecordNoAsync(re,
                    JsonSerializer.Serialize(result));
                    
                return result;
            }
            catch(OrderDomainException ode) when(ode.InnerException is ValidationException)
            {
                await _requestManager.RemoveRequestAsync(request.TradeNo);
                throw;
            }
        }
    }
}