﻿using System.Text.Json;
using System.Threading;
using System.Threading.Tasks;
using MediatR;
using PXPlus.Payment.Application.Dto;
using PXPlus.Payment.Domain.DomainExceptions;
using PXPlus.Payment.Domain.DomainServices.Interface;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Infrastructure.Idempotency;

namespace PXPlus.Payment.Application.Commands
{
    public class OrderConfirmCommandHandler : IRequestHandler<OrderConfirmCommand, PaymentAppResponse>
    {
        private readonly IOrderRepository _orderRepository;
        private readonly IMediator _mediator;
        private readonly IPayToolDomainService _payToolDomainService;

        public OrderConfirmCommandHandler(IOrderRepository orderRepository, IMediator mediator, IPayToolDomainService payToolDomainService)
        {
            _orderRepository = orderRepository;
            _mediator = mediator;
            _payToolDomainService = payToolDomainService;
        }
        
        public async Task<PaymentAppResponse> Handle(OrderConfirmCommand request, CancellationToken cancellationToken)
        {
            var order = await _orderRepository.GetOrderByAsync(request.PXTradeNo);

            if (order.PayerInfo.PayerId != request.PayerId)
                throw new OrderDomainException(PaymentStatusCode.OrderConfirmFailed,"買家與原單不符");

            if (order.ReceiveStoreInfo.StoreCode != request.StoreCode)
                throw new OrderDomainException(PaymentStatusCode.OrderConfirmFailed,"店家與原單不符");

            var orderPaymentCommand = new OrderPaymentCommand(order);

            // re-get pay tool
            var payTool = await _payToolDomainService.GetPayToolAsync(order.PayerInfo.PayToolType, order.PayerInfo.PayerId,
                order.PayerInfo.Sequence);
            
            order.PayerInfo.SetPayTool(payTool);
            
            return await _mediator.Send(orderPaymentCommand, cancellationToken);
        }
    }

    public class
        OrderConfirmIdentifiedCommandHandler : IdentifiedCommandHandler<OrderConfirmCommand, PaymentAppResponse>
    {
        private readonly IRequestManager _requestManager;

        public OrderConfirmIdentifiedCommandHandler(IMediator mediator, IRequestManager requestManager) : base(mediator,
            requestManager)
        {
            _requestManager = requestManager;
        }

        protected override async Task<PaymentAppResponse> CreateResultForDuplicateRequestAsync(
            IdentifiedCommand<OrderConfirmCommand, PaymentAppResponse> request)
        {
            var query = await _requestManager.GetAsync(request.TradeNo);

            if (query.ClientRequestType != ClientRequestType.OrderConfirmCommand)
                throw new OrderDomainException(PaymentStatusCode.TradeNoHasBeenUsed);

            if (query.ResponseData == null)
                throw new OrderDomainException(PaymentStatusCode.TradeProcessing);

            return JsonSerializer.Deserialize<PaymentAppResponse>(query.ResponseData);
        }
    }
}