﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using MediatR;
using PXPlus.Payment.Application.Dto;
using PXPlus.Payment.Domain.Factories.Interface;
using PXPlus.Payment.Domain.OrderAggregate;

namespace PXPlus.Payment.Application.Commands
{
    public class OrderPaymentCommandHandler : IRequestHandler<OrderPaymentCommand, PaymentAppResponse>
    {
        private readonly IExecutePaymentDomainServiceFactory _executePaymentDomainServiceFactory;
        private readonly IOrderRepository _orderRepository;

        public OrderPaymentCommandHandler(IExecutePaymentDomainServiceFactory executePaymentDomainServiceFactory, IOrderRepository orderRepository)
        {
            _executePaymentDomainServiceFactory = executePaymentDomainServiceFactory;
            _orderRepository = orderRepository;
        }
        
        public async Task<PaymentAppResponse> Handle(OrderPaymentCommand request, CancellationToken cancellationToken)
        {
            var order = request.Order;

            var paymentDomainService = _executePaymentDomainServiceFactory.Build(order.PayerInfo.PayToolType);

            var accountPayment = await paymentDomainService.PaymentAsync(
                order.OrderNo.GetOrderNo(),
                order.PayAmount,
                order.PayerInfo.GetPayTool(),
                order.ReceiveStoreInfo);

            order.PaymentComplete(accountPayment.IsSuccess);

            //todo fake back info
            order.SetCashBackAndPointBack(new BackInfo());
            await _orderRepository.UnitOfWork.SaveEntitiesAsync(cancellationToken);

            var response = new PaymentAppResponse
            {
                Code = accountPayment.IsSuccess ? "0000" : accountPayment.Code,
                Message = accountPayment.IsSuccess ? "交易成功" : accountPayment.Message,
                PXTradeNo = order.OrderNo.GetOrderNo(),
                OrderAmount = order.OrderAmount,
                PayAmount = order.PayAmount,
                DiscountPoint = order.PointDiscountInfo.ActualDiscountAmount,
                PayToolType = order.PayerInfo.PayToolType,
                PayToolName = order.PayerInfo.GetPayTool().GetPayToolName(),
                PayToolIdentity = order.PayerInfo.GetMaskedPayerPaymentAccount(),
                Activities = order.ActivityCashDiscountInfos.Select(c => new Activity
                {
                    ActivityCode = c.ActivityId,
                    ProductAmount = (double) c.ProductAmount,
                    DiscountAmount = (double) c.CashDiscountAmount
                }),
                NoneFeedbackAmount = order.NoneFeedbackAmount,
                NoneDiscountAmount = order.NoneDiscountAmount,
                TransactionTime = order.GetCreatedTime,
                StoreName = order.ReceiveStoreInfo.StoreName,
                MerchantName = order.ReceiveStoreInfo.MerchantName,
                PayerEpAccount = order.PayerInfo.PayerEpAccount,
                StoreEpAccount = order.ReceiveStoreInfo.ReceiveAccount,
                PayerId = order.PayerInfo.PayerId
            };

            return response;
        }
    }
}