﻿using System.Linq;
using System.Text.Json;
using System.Threading;
using System.Threading.Tasks;
using MediatR;
using Microsoft.Extensions.Options;
using PXPlus.Core.Extensions;
using PXPlus.Payment.Application.Configs;
using PXPlus.Payment.Application.Dto;
using PXPlus.Payment.Domain.DomainExceptions;
using PXPlus.Payment.Domain.DomainServices.Interface;
using PXPlus.Payment.Domain.Factories.Interface;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Domain.OrderAggregate.PayTools;
using PXPlus.Payment.Domain.PaymentQuotaAggregate;
using PXPlus.Payment.Infrastructure.Idempotency;

namespace PXPlus.Payment.Application.Commands
{
    public class PaymentCommandHandler : IRequestHandler<PaymentCommand, PaymentAppResponse>
    {
        private readonly IMerchantDomainService _merchantDomainService;
        private readonly IOrderRepository _orderRepository;
        private readonly IOrderFactory _orderFactory;
        private readonly IPaymentQuotaDomainService _paymentQuotaDomainService;
        
        private readonly PaymentQuotaLimitConfig _paymentQuotaLimitConfig;
        private readonly IMediator _mediator;
        private readonly IPayToolDomainService _payToolDomainService;
        private readonly IPointDomainService _pointDomainService;
        private readonly IActivityDomainService _activityDomainService;

        public PaymentCommandHandler(IMerchantDomainService merchantDomainService,
            IOrderRepository orderRepository,
            IOrderFactory orderFactory,
            IPaymentQuotaDomainService paymentQuotaDomainService,
            IOptionsMonitor<PaymentQuotaLimitConfig> paymentQuotaLimitConfig,
            IMediator mediator,
            IPayToolDomainService payToolDomainService,
            IPointDomainService pointDomainService,
            IActivityDomainService activityDomainService)
        {
            _merchantDomainService = merchantDomainService;
            _orderRepository = orderRepository;
            _orderFactory = orderFactory;
            _paymentQuotaDomainService = paymentQuotaDomainService;
            _paymentQuotaLimitConfig = paymentQuotaLimitConfig.CurrentValue;
            _mediator = mediator;
            _payToolDomainService = payToolDomainService;
            _pointDomainService = pointDomainService;
            _activityDomainService = activityDomainService;
        }

        public async Task<PaymentAppResponse> Handle(PaymentCommand request, CancellationToken cancellationToken)
        {
            var storeInfo = await _merchantDomainService.GetStoreAsync(request.StoreCode,(int)request.PayToolType);

            var paymentQuota = await _paymentQuotaDomainService.VerifyPaymentQuotaAndReturnQuota(request.PayerId,
                storeInfo.StoreMemberId,
                request.OrderAmount);

            var payerInfoTask = GetPayerInfoAsync(request.PayerId, request.PayToolType, request.Sequence,request.PhoneNumber);

            var receivePayToolTask = _payToolDomainService.GetPayToolAsync(
                PayToolType.EPAccount,
                storeInfo.StoreMemberId,
                0);

            var storeReceivePayTool = await receivePayToolTask;

            storeInfo.SetReceiveAccount(storeReceivePayTool.GetPaymentAccount());
            storeInfo.SetMerchantTradeNo(request.MerchantTradeNo);

            var order = CreateInitialOrder(await payerInfoTask, request, storeInfo);

            await _orderRepository.AddAsync(order);
            
            await _orderRepository.UnitOfWork.SaveEntitiesAsync(cancellationToken);
            
            order.CalculateAvailablePointDiscount();

            if (order.ActivityCashDiscountInfos.Any())
            {
                var activitiesCashDiscount =
                    await _activityDomainService.CalculateActivitiesCashDiscountAsync(order.ActivityCashDiscountInfos);

                order.AddActivityCashDiscountInfo(activitiesCashDiscount);
            }

            if (order.PointDiscountInfo.IsUsePoint)
            {
                var actualDiscountPoint = await _pointDomainService.DeductedToNoneAsync(order.PayerInfo.PayerId,
                    order.PointDiscountInfo.AvailableDiscountAmount, order.OrderNo.GetOrderNo(), order.GetCreatedTime);

                order.PointDiscountInfo.CalculateActualDiscountAmount(actualDiscountPoint);
            }

            order.CalculatePaymentDiscount();

            await _orderRepository.UnitOfWork.SaveEntitiesAsync(cancellationToken);

            if (IsOrderToBeConfirmed(paymentQuota, order.OrderAmount, request.IsEcTrade))
            {
                order.OrderToBeConfirmed();

                await _orderRepository.UnitOfWork.SaveEntitiesAsync(cancellationToken);

                return new PaymentAppResponse
                {
                    PXTradeNo = order.OrderNo.GetOrderNo(),
                    Code = PaymentStatusCode.OrderToBeConfirm.GetResultCode(),
                    Message = PaymentStatusCode.OrderToBeConfirm.GetResulteMessage(),
                    TransactionTime = order.GetCreatedTime,
                    PayToolType = order.PayerInfo.PayToolType,
                    PayToolName = order.PayerInfo.GetPayTool().GetPayToolName(),
                    PayToolIdentity = order.PayerInfo.GetMaskedPayerPaymentAccount(),
                    PayerEpAccount = order.PayerInfo.PayerEpAccount,
                    StoreEpAccount = order.ReceiveStoreInfo.ReceiveAccount,
                    StoreName = order.ReceiveStoreInfo.StoreName,
                    OrderAmount = order.OrderAmount,
                    PayAmount = order.PayAmount,
                    DiscountPoint = order.PointDiscountInfo.ActualDiscountAmount,
                    MerchantName = order.ReceiveStoreInfo.MerchantName,
                    PayerId = order.PayerInfo.PayerId
                };
            }

            var orderPaymentCommand = new OrderPaymentCommand(order);

            return await _mediator.Send(orderPaymentCommand, cancellationToken);
        }

        private async Task<PayerInfo> GetPayerInfoAsync(long payerId, PayToolType payToolType, int sequence,string phoneNumber)
        {
            var getPayerEpAccountTask = _payToolDomainService.GetPayToolAsync(
                PayToolType.EPAccount,
                payerId,
                0);

            Task<IPayTool> getPayerPayToolTask;

            if (payToolType == PayToolType.EPAccount)
                getPayerPayToolTask = Task.FromResult(await getPayerEpAccountTask);
            else
                getPayerPayToolTask = _payToolDomainService.GetPayToolAsync(
                    payToolType,
                    payerId,
                    sequence);

            var payerPayTool = await getPayerPayToolTask;

            var payerEpAccount = (EpAccountPayTool) await getPayerEpAccountTask;

            return new PayerInfo(payerId, payerEpAccount.EpAccount, payToolType, sequence, payerPayTool,phoneNumber);
        }

        private Order CreateInitialOrder(PayerInfo payerInfo, PaymentCommand request, ReceiveStoreInfo storeInfo)
            => _orderFactory.CreateInitialOrder(
                payerInfo
                , request.IsUsePoint
                , request.OrderAmount
                , storeInfo.PointDiscountPercent
                , request.NoneDiscountAmount
                , request.NoneFeedbackAmount
                , storeInfo
                , request.PlatformType
                , request.ActivityCashDiscountInfos.ToList()
                , request.InputMetaData
            );

        private bool IsOrderToBeConfirmed(PaymentQuota paymentQuota, decimal orderAmount, bool isEcTrade)
        {
            if (isEcTrade)
                return true;

            if (orderAmount > _paymentQuotaLimitConfig.LimitEveryTrade)
                return true;

            if (orderAmount + paymentQuota.DailyQuota> _paymentQuotaLimitConfig.LimitEveryDay)
                return true;

            if (orderAmount + paymentQuota.MonthlyQuota > _paymentQuotaLimitConfig.LimitEveryMonth)
                return true;

            return false;
        }
    }


    public class Activity
    {
        public string ActivityCode { get; set; }
        public double ProductAmount { get; set; }
        public double DiscountAmount { get; set; }
    }


    public class PaymentIdentifiedCommandHandler : IdentifiedCommandHandler<PaymentCommand, PaymentAppResponse>
    {
        private readonly IRequestManager _requestManager;

        public PaymentIdentifiedCommandHandler(IMediator mediator, IRequestManager requestManager) : base(mediator,
            requestManager)
        {
            _requestManager = requestManager;
        }

        protected override async Task<PaymentAppResponse> CreateResultForDuplicateRequestAsync(
            IdentifiedCommand<PaymentCommand, PaymentAppResponse> request)
        {
            var query = await _requestManager.GetAsync(request.TradeNo);

            if (query.ClientRequestType != ClientRequestType.PaymentCommand)
                throw new OrderDomainException(PaymentStatusCode.TradeNoHasBeenUsed);

            if (query.ResponseData == null)
                throw new OrderDomainException(PaymentStatusCode.TradeProcessing);

            return JsonSerializer.Deserialize<PaymentAppResponse>(query.ResponseData);
        }
    }
}