using System.Linq;
using System.Text.Json;
using System.Threading;
using System.Threading.Tasks;
using MediatR;
using Microsoft.Extensions.Logging;
using PXPlus.Payment.Application.Dto;
using PXPlus.Payment.Domain.DomainExceptions;
using PXPlus.Payment.Domain.DomainServices.Interface;
using PXPlus.Payment.Domain.Factories.Interface;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Infrastructure.Idempotency;

namespace PXPlus.Payment.Application.Commands
{
    public class RefundCommandHandler : IRequestHandler<RefundCommand, RefundAppResponse>
    {
        private readonly IOrderRepository _orderRepository;
        private readonly IActivityDomainService _activityDomainService;
        private readonly IExecutePaymentDomainServiceFactory _executePaymentDomainServiceFactory;
        private readonly IGenerateOrderNoDomainService _generateOrderNoDomainService;

        public RefundCommandHandler(IOrderRepository orderRepository, IActivityDomainService activityDomainService, IExecutePaymentDomainServiceFactory executePaymentDomainServiceFactory,
            IGenerateOrderNoDomainService generateOrderNoDomainService)
        {
            _orderRepository = orderRepository;
            _activityDomainService = activityDomainService;
            _executePaymentDomainServiceFactory = executePaymentDomainServiceFactory;
            _generateOrderNoDomainService = generateOrderNoDomainService;
        }
        
        public async Task<RefundAppResponse> Handle(RefundCommand request, CancellationToken cancellationToken)
        {
            var order = await _orderRepository.GetOrderByAsync(request.OriginalPxTradeNo);

            if (order == null)
                throw new OrderDomainException(PaymentStatusCode.RefundFailedOrderNotFound);
            
            //lock refund order
            
            var originalPayAmount = order.PayAmount;
            var originalDiscountAmount = order.PointDiscountInfo.ActualDiscountAmount;
            var refundOrder = _generateOrderNoDomainService.GenerateOrderNoWithPrefix(order.OrderNo.Prefix);
            
            //建立initial退單
            order.TransformRefundOrder(
                refundOrder,
                request.RefunderId,
                request.StoreCode,
                request.Amount,
                request.NoneDiscountAmount,
                request.NoneFeedbackAmount,
                request.RefundCause,
                request.ExecuteSource,
                request.Executor);
            
            order.ReceiveStoreInfo.SetMerchantTradeNo(request.MerchantTradeNo);
            
            order.RemoveActivityCashDiscountInfo(request.ActivityCashDiscountInfos);

            var activitiesCashDiscount =
                await _activityDomainService.CalculateActivitiesCashDiscountAsync(order.ActivityCashDiscountInfos);

            order.ClearActivityCashDiscountInfo();
            order.AddActivityCashDiscountInfo(activitiesCashDiscount);


            //recalculate order amount and pay amount
            order.CalculateRefundDiscount(request.Amount);

            var refundPayAmount = originalPayAmount - order.PayAmount;
            var refundPointAmount = originalDiscountAmount - order.PointDiscountInfo.ActualDiscountAmount;

            var executePaymentDomainService = _executePaymentDomainServiceFactory.Build(order.PayerInfo.PayToolType);
            
            var accountRefundResult = await executePaymentDomainService.RefundAsync(
                order.OrderNo.GetOrderNo(), 
                order.RefundOrderNo.GetOrderNo(),
                refundPayAmount, 
                order.PayerInfo, 
                order.ReceiveStoreInfo
                );
            
            //todo recalculate cashback and pointback
            order.SetCashBackAndPointBack(new BackInfo());
            
            order.RefundComplete(accountRefundResult.IsSuccess,request.Amount,refundPayAmount,refundPointAmount,
            request.NoneDiscountAmount,request.NoneFeedbackAmount,0,0);

            //9. publish event
            await _orderRepository.UnitOfWork.SaveEntitiesAsync(cancellationToken);
                
            var response = new RefundAppResponse
                           {
                               Code = accountRefundResult.IsSuccess ? "0000" : accountRefundResult.Code,
                               Message = accountRefundResult.IsSuccess ? "交易成功" : "交易失敗: "+accountRefundResult.Message,
                               PXTradeNo = order.RefundOrderNo.GetOrderNo(),
                               Amount = request.Amount,
                               TradeAmount = refundPayAmount,
                               DiscountPoint = refundPointAmount,
                               Activities = order.ActivityCashDiscountInfos.Select(c=>new Activity
                                   {
                                       ActivityCode = c.ActivityId,
                                       ProductAmount = (double)c.ProductAmount,
                                       DiscountAmount = (double)c.CashDiscountAmount
                                   })
                           };
            return response;
        }
    }

    public class RefundIdentifiedCommandHandler : IdentifiedCommandHandler<RefundCommand,RefundAppResponse>
    {
        private readonly IRequestManager _requestManager;

        public RefundIdentifiedCommandHandler(IMediator mediator, IRequestManager requestManager) : base(mediator, requestManager)
        {
            _requestManager = requestManager;
        }

        protected override async Task<RefundAppResponse> CreateResultForDuplicateRequestAsync(
            IdentifiedCommand<RefundCommand, RefundAppResponse> request)
        {
            var query = await _requestManager.GetAsync(request.TradeNo);
            
            if (query.ClientRequestType != ClientRequestType.RefundCommand)
                throw new OrderDomainException(PaymentStatusCode.TradeNoHasBeenUsed);
            
            if (query.ResponseData == null)
                throw new OrderDomainException(PaymentStatusCode.TradeProcessing);
            
            return JsonSerializer.Deserialize<RefundAppResponse>(query.ResponseData);
        }
    }
}