using System.Threading;
using System.Threading.Tasks;
using MediatR;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using PXPlus.Core.IntegrationEventLogEf.Services;
using PXPlus.Payment.Application.IntegrationEvents.Events;
using PXPlus.Payment.Domain.DomainServices.Interface;
using PXPlus.Payment.Domain.Events;
using PXPlus.Payment.Domain.OrderRecordAggregate;

namespace PXPlus.Payment.Application.DomainEventHandlers.OrderRefund
{
    public class OrderRefundSuccessfulDomainEventHandler : INotificationHandler<OrderRefundSuccessfulDomainEvent>
    {
        private readonly IOrderRecordRepository _orderRecordRepository;
        private readonly IPointDomainService _pointDomainService;
        private readonly ILogger<OrderRefundSuccessfulDomainEventHandler> _logger;
        private readonly IIntegrationEventService _integrationEventService;
        private readonly IHttpContextAccessor _httpContextAccessor;
        public OrderRefundSuccessfulDomainEventHandler(IOrderRecordRepository orderRecordRepository,
            IPointDomainService pointDomainService, ILogger<OrderRefundSuccessfulDomainEventHandler> logger,
            IIntegrationEventService integrationEventService,IHttpContextAccessor httpContextAccessor)
        {
            _orderRecordRepository = orderRecordRepository;
            _pointDomainService = pointDomainService;
            _logger = logger;
            _integrationEventService = integrationEventService;
            _httpContextAccessor = httpContextAccessor;
        }

        public async Task Handle(OrderRefundSuccessfulDomainEvent notification, CancellationToken cancellationToken)
        {
            var refundOrder = notification.Order;

            var orderRecord = OrderRecord.FromOrderDomainModelForRefundOrder(notification.Order
                , notification.Order.RefundOrderNo.GetOrderNo()
                , notification.RefundOrderAmount
                , notification.RefundPayAmount
                , notification.RefundPointAmount
                , notification.NoneDiscountAmount
                , notification.NoneFeedbackAmount
            );

            orderRecord.SetRecordTypeAndResult(OrderRecordResult.Success);

            if (notification.RefundPointAmount > 0)
                await _pointDomainService.ReturnPointAsync(notification.RefundPointAmount);

            await _orderRecordRepository.AddAsync(orderRecord);

            await _orderRecordRepository.UnitOfWork.SaveEntitiesAsync(cancellationToken);

            var xRequestId = _httpContextAccessor.HttpContext.TraceIdentifier;
            
            var orderRefundSuccessfulIntegrationEvent = new OrderRefundSuccessfulIntegrationEvent(
                refundOrder,orderRecord, xRequestId);

            var paymentRecordIntegrationEvent = new PaymentRecordIntegrationEvent(
                orderRefundSuccessfulIntegrationEvent, notification.Order.PayerInfo.PayerId, xRequestId);
            
            await _integrationEventService.AddAndSaveEventAsync(orderRefundSuccessfulIntegrationEvent);
            await _integrationEventService.AddAndSaveEventAsync(paymentRecordIntegrationEvent);
        }
    }
}