﻿using System.Text.Json;
using System.Threading.Tasks;
using PXPlus.Payment.Application.Dto;
using PXPlus.Payment.Domain.DomainExceptions;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Infrastructure.Idempotency;

namespace PXPlus.Payment.Application.Queries
{
    public class QueryAppService
    {
        private readonly IRequestManager _requestManager;
        private readonly IOrderRepository _orderRepository;

        public QueryAppService(IRequestManager requestManager, IOrderRepository orderRepository)
        {
            _requestManager = requestManager;
            _orderRepository = orderRepository;
        }

        public async Task<QueryAppResponse> QueryAsync(string tradeNo)
        {
            if (string.IsNullOrEmpty(tradeNo))
                throw new OrderDomainException(PaymentStatusCode.InvalidParameter);

            var clientRequest = await _requestManager.GetAsync(tradeNo);
            
            if (clientRequest == null)
                return new QueryAppResponse
                       {
                           OrderQueryType = OrderQueryType.OrderNotFound
                       };

            if (clientRequest.ResponseData == null)
                throw new OrderDomainException(PaymentStatusCode.TradeProcessing);
            
            var recordType = clientRequest.ClientRequestType;

            var queryResponse = new QueryAppResponse();

            switch (recordType)
            {
                case ClientRequestType.PaymentCommand:
                    queryResponse.OrderQueryType = OrderQueryType.PaymentOrder;
                    queryResponse.PaymentAppResponse = JsonSerializer.Deserialize<PaymentAppResponse>(clientRequest.ResponseData);
                    break;
                case ClientRequestType.OrderConfirmCommand:
                    queryResponse.OrderQueryType = OrderQueryType.PaymentOrder;
                    queryResponse.PaymentAppResponse = JsonSerializer.Deserialize<PaymentAppResponse>(clientRequest.ResponseData);
                    break;
                case ClientRequestType.RefundCommand:
                    queryResponse.OrderQueryType = OrderQueryType.RefundOrder;
                    queryResponse.RefundAppResponse =
                        JsonSerializer.Deserialize<RefundAppResponse>(clientRequest.ResponseData);
                    break;
                default:
                    queryResponse.OrderQueryType = OrderQueryType.OrderNotFound;
                    break;
            }

            return queryResponse;
        }
        
        
        public async Task<QueryAppResponse> QueryWithPxTradeNoAsync(string pxTradeNo)
        {
            if (string.IsNullOrEmpty(pxTradeNo))
                throw new OrderDomainException(PaymentStatusCode.InvalidParameter);

            var order = await _orderRepository.GetOrderByNoTrackingAsync(pxTradeNo);
            
            if (order == null)
                return new QueryAppResponse
                {
                    OrderQueryType = OrderQueryType.OrderNotFound
                };

            return new QueryAppResponse
            {
                OrderQueryType = OrderQueryType.PaymentOrder,
                PaymentAppResponse = new PaymentAppResponse
                {
                    PXTradeNo = order.OrderNo.GetOrderNo(),
                    OrderAmount = order.OrderAmount,
                    PayAmount = order.PayAmount,
                    DiscountPoint = order.PointDiscountInfo.ActualDiscountAmount,
                    PayToolType = order.PayerInfo.PayToolType,
                    PayToolName = order.PayerInfo.PayToolType.ToString(),
                    PayToolIdentity = order.PayerInfo.PayerPaymentAccount,
                    NoneFeedbackAmount = order.NoneFeedbackAmount,
                    NoneDiscountAmount = order.NoneDiscountAmount,
                    TransactionTime = order.GetCreatedTime,
                    PayerEpAccount = order.PayerInfo.PayerEpAccount,
                    StoreEpAccount = order.ReceiveStoreInfo.ReceiveAccount,
                    PayerId = order.PayerInfo.PayerId,
                    StoreName = order.ReceiveStoreInfo.StoreName,
                    MerchantName = order.ReceiveStoreInfo.MerchantName
                }
            };

        }
    }

    public enum OrderQueryType
    {
        OrderNotFound,
        PaymentOrder,
        RefundOrder
    }
}