﻿using System;
using System.Collections.Generic;
using System.Linq;
using PXPlus.Enum.Authorization;
using PXPlus.Payment.Domain.DomainExceptions;
using PXPlus.Payment.Domain.Events;
using PXPlus.Payment.Domain.SeedWork;

namespace PXPlus.Payment.Domain.OrderAggregate
{
    public partial class Order : Entity, IAggregateRoot
    {
        
        /// <summary>
        /// 訂單號
        /// </summary>
        public OrderNo OrderNo { get; private set; }
        
        /// <summary>
        /// 退款訂單號
        /// </summary>
        public OrderNo RefundOrderNo { get;private set; }
        
        /// <summary>
        /// 訂單金額
        /// </summary>
        public decimal OrderAmount { get; private set; }

        /// <summary>
        /// 支付金額
        /// </summary>
        public decimal PayAmount { get; private set; }

        /// <summary>
        /// 不可折抵金額
        /// </summary>
        public decimal NoneDiscountAmount { get; private set; }
        
        /// <summary>
        /// 不可回饋金額
        /// </summary>
        public decimal NoneFeedbackAmount { get; private set; }
        
        /// <summary>
        /// 訂單狀態
        /// </summary>
        public OrderStatus OrderStatus { get; private set; }
        
        /// <summary>
        /// 買家資訊
        /// </summary>
        public PayerInfo PayerInfo { get; private set; }

        /// <summary>
        /// 店家資訊
        /// </summary>
        public ReceiveStoreInfo ReceiveStoreInfo { get; private set; }

        /// <summary>
        /// 點數折抵
        /// </summary>
        public PointDiscountInfo PointDiscountInfo { get;private set; }

        /// <summary>
        /// 活動現金折抵
        /// </summary>
        public IReadOnlyCollection<ActivityCashDiscountInfo> ActivityCashDiscountInfos => _activityCashDiscountInfos;

        /// <summary>
        /// 活動回饋
        /// </summary>
        public IReadOnlyCollection<ActivityFeedbackInfo> ActivityFeedbackInfos => _activityFeedbackInfos;

        public BackInfo BackInfo { get; private set; }
        public int RefundCount { get; private set; }

        public PlatformType PlatformType { get; private set; }

        public CurrencyType CurrencyType { get; private set; }

        public MetaData InputMetaData { get; private set; }
        public MetaData ReceiveMetaData { get; private set; }
        public string BankReturnCode { get;private set; }

        public string RefundCause { get; private set; }
        public int ExecuteSource { get; private  set; }
        public string Executor { get; private set; }
        public DateTime GetUpdatedTime => _updatedTime;

        public DateTime GetCreatedTime => _createdTime;
        public void SetInputMetaData(MetaData metaData)
        {
            InputMetaData = metaData;
        }

        public void SetReceiveMetaData(MetaData metaData)
        {
            ReceiveMetaData = metaData;
        }
        
        private decimal GetAvailableDiscountAmount()
        {
            if (OrderAmount - NoneDiscountAmount <0)
                return 0;
            
            return OrderAmount - NoneDiscountAmount;
        }

        public decimal GetAvailableFeedbackAmount()
        {
            if (OrderAmount - NoneFeedbackAmount < 0)
                return 0;
            
            return OrderAmount - NoneFeedbackAmount;
        }
            
        public void AddActivityFeedbackInfo(IEnumerable<ActivityFeedbackInfo> feedbackInfos)
        {
            _activityFeedbackInfos.AddRange(feedbackInfos);
        }

        public void ClearActivityCashDiscountInfo()
        {
            _activityCashDiscountInfos.Clear();
        }
        
        public void AddActivityCashDiscountInfo(IEnumerable<ActivityCashDiscountInfo> cashDiscountInfos)
        {
            _activityCashDiscountInfos.AddRange(cashDiscountInfos);
        }
        
        public void RemoveActivityFeedbackInfo(IEnumerable<ActivityFeedbackInfo> feedbackInfos)
        {
            foreach (var info in feedbackInfos)
                _activityFeedbackInfos.Remove(info);
        }

        public void RemoveActivityCashDiscountInfo(IEnumerable<ActivityCashDiscountInfo> cashDiscountInfos)
        {
            if (_activityCashDiscountInfos==null || !_activityCashDiscountInfos.Any())
                return;

            _activityCashDiscountInfos.RemoveAll(
                c => cashDiscountInfos.Select(s => s.ActivityId).Contains(c.ActivityId));
        }

        
        public void CalculateAvailablePointDiscount()
        {
            PointDiscountInfo.CalculateAvailableDiscount(GetAvailableDiscountAmount());
        }

        public void CalculatePaymentDiscount()
        {
            PayAmount = OrderAmount
                        - PointDiscountInfo.ActualDiscountAmount
                        - (ActivityCashDiscountInfos?.Sum(c=>c.CashDiscountAmount)??0);
        }

        public void SetCashBackAndPointBack(BackInfo backInfo)
        {
            BackInfo = backInfo;
        }
        
        public decimal GetCashFeedBack()
            => ActivityFeedbackInfos?.Sum(c => c.CashFeedback)??0;

        public decimal GetPointFeedBack()
            => ActivityFeedbackInfos?.Sum(c => c.PointFeedback)??0;

        public void PaymentComplete(bool isSuccess)
        {
            _updatedTime = DateTime.Now;

            if (isSuccess)
            {
                OrderStatus = OrderStatus.PaymentSuccess;
                AddDomainEvent(new OrderPaymentSuccessfulDomainEvent(this));
            }
            else
            {
                OrderStatus = OrderStatus.PaymentFailed;
                AddDomainEvent(new OrderPaymentFailedDomainEvent(this));
            }

        }

        public void OrderToBeConfirmed()
        {
            OrderStatus = OrderStatus.PaymentTobeConfirm;
            _updatedTime = DateTime.Now;
        }

        public void PaymentCancel()
        {
            OrderStatus = OrderStatus.PaymentCancel;
            _updatedTime = DateTime.Now;
            AddDomainEvent(new OrderPaymentCanceledDomainEvent(this));    
        }

        private void DetermineCanRefund(long refunderId, string storeCode, decimal refundAmount)
        {
            if (! PayerInfo.PayerId.Equals(refunderId))
                throw new OrderDomainException(PaymentStatusCode.RefundFailed,"付款者及退款者不同!");
            
            if (! ReceiveStoreInfo.StoreCode.Equals(storeCode))
                throw new OrderDomainException(PaymentStatusCode.RefundFailed,"付款店家及收款店家不同!");
            
            if (! (OrderStatus == OrderStatus.PaymentSuccess || OrderStatus == OrderStatus.Refunded))
                throw new OrderDomainException(PaymentStatusCode.RefundFailed,"訂單狀態錯誤!");

            if (refundAmount > OrderAmount)
                throw new OrderDomainException(PaymentStatusCode.RefundFailedAvailableAmountNotEnough);
            
        }

        public void CalculateRefundDiscount(decimal refundAmount)
        {
            OrderAmount -= refundAmount;
            PointDiscountInfo.CalculateAvailableDiscount(GetAvailableDiscountAmount());
            PointDiscountInfo.CalculateActualDiscountAmount(PointDiscountInfo.ActualDiscountAmount);
            CalculatePaymentDiscount();
        }

        public void TransformRefundOrder(OrderNo orderNo,long refunderId, string storeCode, decimal refundAmount,decimal noneDiscountAmount ,decimal noneFeedbackAmount
        ,string refundCause,int executeSource,string executor)
        {
            DetermineCanRefund(refunderId,storeCode,refundAmount);
            RefundOrderNo = orderNo;
            
            if (noneDiscountAmount > refundAmount)
                noneDiscountAmount = refundAmount;

            if (noneFeedbackAmount > refundAmount)
                noneDiscountAmount = refundAmount;
            
            NoneDiscountAmount -= noneDiscountAmount;
            NoneFeedbackAmount -= noneFeedbackAmount;
            
            RefundCause = refundCause;
            ExecuteSource = executeSource;
            Executor = executor;
        }
        

        public void RefundComplete(bool isSuccess,decimal refundOrderAmount,decimal refundPayAmount,decimal refundPoint,
            decimal noneDiscountAmount,decimal noneFeedbackAmount,decimal cashBack,decimal pointBack)
        {
            _updatedTime = DateTime.Now;
            RefundCount++;    
            
            if (isSuccess)
            {
                OrderStatus = OrderStatus.Refunded;
                AddDomainEvent(new OrderRefundSuccessfulDomainEvent(this,
                    refundOrderAmount,
                    refundPayAmount,
                    refundPoint,
                    noneDiscountAmount,
                    noneFeedbackAmount,
                    cashBack,
                    pointBack));
            }
            else
            {
                AddDomainEvent(new OrderRefundFailedDomainEvent(this));
            }
        }
    }
    
    /// <summary>
    /// For persistence
    /// </summary>
    public partial class Order
    {
        protected Order()
        {
        }

        private readonly List<ActivityFeedbackInfo> _activityFeedbackInfos ;
        
        
        private readonly List<ActivityCashDiscountInfo> _activityCashDiscountInfos;
        /// <summary>
        /// 建立時間
        /// </summary>
        private DateTime _createdTime;
        /// <summary>
        /// 更新時間
        /// </summary>
        private DateTime _updatedTime;
        
    }

    public partial class Order
    {
        //For Processing Order
        public Order(OrderNo orderNo,
            PayerInfo payerInfo,
            ReceiveStoreInfo receiveStoreInfo, decimal orderAmount, decimal noneDiscountAmount,
            decimal noneFeedbackAmount, PointDiscountInfo pointDiscountInfo,
            PlatformType platformType,
            List<ActivityCashDiscountInfo> activityCashDiscountInfos)
        {
            OrderNo = orderNo;
            PayerInfo = payerInfo;
            ReceiveStoreInfo = receiveStoreInfo;
            OrderAmount = orderAmount;
            NoneDiscountAmount = noneDiscountAmount;
            NoneFeedbackAmount = noneFeedbackAmount;
            PointDiscountInfo = pointDiscountInfo;
            PlatformType = platformType;
            _activityCashDiscountInfos = activityCashDiscountInfos;
            _createdTime = DateTime.Now;
            _updatedTime = DateTime.Now;
            OrderStatus = OrderStatus.PaymentProgressing;
            //default = NTD
            CurrencyType = CurrencyType.NTD;
            _activityFeedbackInfos = new List<ActivityFeedbackInfo>();
        }
    }

    public enum OrderStatus
    {
        PaymentProgressing,
        PaymentSuccess,
        PaymentFailed,
        PaymentTobeConfirm,
        PaymentCancel,
        RefundProgressing,
        Refunded
    }
}