using System;
using System.Threading.Tasks;
using AccountForPaymentService;
using AccountLinkService;
using Microsoft.Extensions.Logging;
using PXPlus.Core.Extensions;
using PXPlus.Payment.Domain.DomainExceptions;
using PXPlus.Payment.Domain.DomainServices.Interface;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Domain.OrderAggregate.PayTools;

namespace PXPlus.Payment.Infrastructure.DomainServiceGrpc.ExecutePaymentDomainServices
{
    public class ExecuteAcLinkPaymentGrpcDomainService : IExecutePaymentDomainService
    {

        private readonly ILogger<ExecuteAcLinkPaymentGrpcDomainService> _logger;
        private readonly AccountLinkGrpcService.AccountLinkGrpcServiceClient _client;
        private readonly EpAccountForPaymentGrpcService.EpAccountForPaymentGrpcServiceClient _epAccountForPaymentGrpcServiceClient;

        public ExecuteAcLinkPaymentGrpcDomainService(ILogger<ExecuteAcLinkPaymentGrpcDomainService> logger, 
            AccountLinkGrpcService.AccountLinkGrpcServiceClient client, 
            EpAccountForPaymentGrpcService.EpAccountForPaymentGrpcServiceClient epAccountForPaymentGrpcServiceClient)
        {
            _logger = logger;
            _client = client;
            _epAccountForPaymentGrpcServiceClient = epAccountForPaymentGrpcServiceClient;
        }
        
        public async Task<ExecutePaymentInfo> PaymentAsync(string pxTradeNo, decimal payAmount, IPayTool payTool,
            ReceiveStoreInfo storeInfo)
        {
            try
            {
                
                if (! (payTool is AccountLinkPayTool accountLinkPayTool))
                    throw new ArgumentNullException(nameof(AccountLinkPayTool));

                var request = new AccountLinkPayGrpcRequest
                {
                    TradeNo = pxTradeNo,
                    Amount = (double)payAmount,
                    BackCode = accountLinkPayTool.BankCode,
                    EpAccount = accountLinkPayTool.EPAccount,
                    VirtualAccount = accountLinkPayTool.VirtualAccount,
                    IdentityNo = accountLinkPayTool.IdentityNo,
                    MerchantType = storeInfo.IndustryType
                };
                
                var response = await _client.AccountLinkPayCommandAsync(request);

                var accountPaymentInfo = new ExecutePaymentInfo
                {
                    IsSuccess = false,
                    Message = response.Message,
                    Code = PaymentStatusCode.AccountLinkPayFailed.GetResultCode(),
                    BankReturnCode = response.Data?.BankReturnCode
                };
                
                if (response.Code == "0000")
                    accountPaymentInfo.IsSuccess = true;

                return accountPaymentInfo;
            }
            catch (Exception e)
            {
                _logger.LogError(e,e.Message);
                return new ExecutePaymentInfo
                       {   
                           IsSuccess = false,
                           Message = e.Message,
                           Code = PaymentStatusCode.AccountLinkPayFailed.GetResultCode()
                       };
            }
        }

        public async Task<ExecuteRefundInfo> RefundAsync(string pxOriTradeNo, string pxRefundTradeNo, decimal payAmount,
            PayerInfo payerInfo, ReceiveStoreInfo storeInfo)
        {
            try
            {
                var response = await _epAccountForPaymentGrpcServiceClient.AccountLinkRefundAsync(
                    new AccountLinkRefundGrpcRequest
                    {
                        TradeNo = pxOriTradeNo,
                        RefundTradeNo = pxRefundTradeNo,
                        RefunderEpAccount = payerInfo.PayerEpAccount,
                        RefundReceiverEpAccount = storeInfo.ReceiveAccount,
                        Amount = (float) payAmount

                    });

                var accountRefundInfo = new ExecuteRefundInfo
                {
                    IsSuccess = false,
                    Message = response.Message,
                    Code = PaymentStatusCode.RefundFailed.GetResultCode(),
                    BankReturnCode = response.Code
                };

                if (response.Code == "0000")
                {
                    accountRefundInfo.IsSuccess = true;
                    accountRefundInfo.Code = PaymentStatusCode.TradeSuccess.GetResultCode();
                }

                return accountRefundInfo;
            }
            catch (Exception e)
            {
                _logger.LogError(e, e.Message);
                return new ExecuteRefundInfo
                {
                    IsSuccess = false,
                    Message = e.Message,
                    Code = PaymentStatusCode.RefundFailed.GetResultCode()
                };
            }
        }
    }
}