﻿using System;
using System.Threading.Tasks;
using AccountForPaymentService;
using Microsoft.Extensions.Logging;
using PXPlus.Payment.Domain.DomainExceptions;
using PXPlus.Payment.Domain.DomainServices.Interface;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Domain.OrderAggregate.PayTools;

namespace PXPlus.Payment.Infrastructure.DomainServiceGrpc
{
    public class PayToolGrpcDomainService : IPayToolDomainService
    {
        private readonly CreditCardPayToolForPaymentGrpcService.CreditCardPayToolForPaymentGrpcServiceClient
            _creditCardPayToolForPaymentGrpcServiceClient;

        private readonly AccountLinkPayToolForPaymentGrpcService.AccountLinkPayToolForPaymentGrpcServiceClient
            _accountLinkPayToolForPaymentGrpcServiceClient;

        private readonly EpAccountPayToolForPaymentGrpcService.EpAccountPayToolForPaymentGrpcServiceClient
            _epAccountPayToolForPaymentGrpcServiceClient;

        private readonly ILogger<PayToolGrpcDomainService> _logger;

        public PayToolGrpcDomainService(
            CreditCardPayToolForPaymentGrpcService.CreditCardPayToolForPaymentGrpcServiceClient
                creditCardPayToolForPaymentGrpcServiceClient,
            AccountLinkPayToolForPaymentGrpcService.AccountLinkPayToolForPaymentGrpcServiceClient
                accountLinkPayToolForPaymentGrpcServiceClient,
            EpAccountPayToolForPaymentGrpcService.EpAccountPayToolForPaymentGrpcServiceClient
                epAccountPayToolForPaymentGrpcServiceClient, ILogger<PayToolGrpcDomainService> logger)
        {
            _creditCardPayToolForPaymentGrpcServiceClient = creditCardPayToolForPaymentGrpcServiceClient;
            _accountLinkPayToolForPaymentGrpcServiceClient = accountLinkPayToolForPaymentGrpcServiceClient;
            _epAccountPayToolForPaymentGrpcServiceClient = epAccountPayToolForPaymentGrpcServiceClient;
            _logger = logger;
        }

        public async Task<IPayTool> GetPayToolAsync(PayToolType payToolType, long payerId, int sequence)
        {
            switch (payToolType)
            {
                case PayToolType.AccountLink:
                    return await GetAccountLinkPayToolAsync(payerId, sequence);
                case PayToolType.CreditCard:
                    return await GetCreditCardToolAsync(payerId, sequence);
                case PayToolType.EPAccount:
                    return await GetEpAccountPayToolAsync(payerId);
                default:
                    throw new ArgumentOutOfRangeException(nameof(payToolType), payToolType, null);
            }
        }

        private async Task<CreditCardPayTool> GetCreditCardToolAsync(long payerId,
            int sequence)
        {
            try
            {
                var cardPayToolResponse = await _creditCardPayToolForPaymentGrpcServiceClient.GetCreditCardPayToolAsync(
                    new GetCreditCardPayToolGrpcRequest
                    {
                        MemberId = payerId,
                        Sequence = sequence
                    });

                if (cardPayToolResponse.Code != "0000")
                    throw new OrderDomainException(cardPayToolResponse.Code, cardPayToolResponse.Message);

                var creditCardData = cardPayToolResponse.Data;

                return new CreditCardPayTool(
                    creditCardData.CreditCard,
                    creditCardData.ExpiryDate,
                    creditCardData.BankName,
                    creditCardData.BankCode,
                    creditCardData.NickName);
            }
            catch (Exception e)
            {
                _logger.LogError(e, e.Message);
                throw new OrderDomainException(PaymentStatusCode.OtherError);
            }
        }

        private async Task<AccountLinkPayTool> GetAccountLinkPayToolAsync(long payerId, int sequence)
        {
            try
            {
                var acLinkPayToolAsync =
                    await _accountLinkPayToolForPaymentGrpcServiceClient.GetAccountLinkPayToolAsync(
                        new GetAccountLinkPayToolGrpcRequest()
                        {
                            MemberId = payerId,
                            Sequence = sequence
                        });

                if (acLinkPayToolAsync.Code != "0000")
                    throw new OrderDomainException(acLinkPayToolAsync.Code, acLinkPayToolAsync.Message);

                var acLinkData = acLinkPayToolAsync.Data;

                return new AccountLinkPayTool(
                    acLinkData.VirtualAccount,
                    acLinkData.EpAccount,
                    acLinkData.IdentityNo,
                    acLinkData.BankName,
                    acLinkData.BankCode,
                    acLinkData.NickName);
            }
            catch (Exception e)
            {
                _logger.LogError(e, e.Message);
                throw new OrderDomainException(PaymentStatusCode.OtherError);
            }
        }

        private async Task<EpAccountPayTool> GetEpAccountPayToolAsync(long memberId)
        {
            try
            {
                var epAccountPayTool = await _epAccountPayToolForPaymentGrpcServiceClient.GetEpAccountPayToolAsync(
                    new GetEpAccountPayToolGrpcRequest
                    {
                        MemberId = memberId,
                        // hard code 0 
                        Sequence = 0
                    });

                if (epAccountPayTool.Code != "0000")
                    throw new OrderDomainException(epAccountPayTool.Code, epAccountPayTool.Message);

                return new EpAccountPayTool(epAccountPayTool.Data.EpAccount);
            }
            catch (Exception e)
            {
                _logger.LogError(e, e.Message);
                throw new OrderDomainException(PaymentStatusCode.OtherError);
            }
        }
    }
}