using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.Json;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.ChangeTracking;
using Microsoft.EntityFrameworkCore.Metadata.Builders;
using PXPlus.Core.Security;
using PXPlus.Payment.Domain.OrderAggregate;

namespace PXPlus.Payment.Infrastructure.EntityConfigurations
{
    public class OrderEntityTypeConfiguration : IEntityTypeConfiguration<Order>
    {
        public void Configure(EntityTypeBuilder<Order> builder)
        {
            builder.ToTable("Orders");
            
            builder.HasKey(c => c.Id);

            builder.OwnsOne(c=>c.OrderNo, z =>
                   {
                          z.HasIndex(c => c.SerialNumber).IsUnique();
                   });
            
            builder.OwnsOne(c=>c.OrderNo)
                   .Property(c=>c.SerialNumber)
                   .HasColumnName("OrderNoSerialNumber")
                   .HasColumnType("char(18)");
            
            builder.OwnsOne(c => c.OrderNo)
                   .Property(c=>c.Prefix)
                   .HasColumnName("OrderNoPrefix")
                   .HasColumnType("char(2)");

            builder.Property<DateTime>("_createdTime")
                   .HasColumnName("CreatedTime")
                   .HasColumnType("DateTime2");

            builder.Property<DateTime>("_updatedTime")
                   .HasColumnName("UpdatedTime")
                   .HasColumnType("DateTime2");

            builder.OwnsOne(c => c.PayerInfo)
                   .Property(c => c.PayerId)
                   .HasColumnName("PayerId")
                   .HasColumnType("bigint");

            builder.OwnsOne(c => c.PayerInfo)
                   .Property(c => c.PayerPaymentAccount)
                   .HasColumnName("PayerAccount")
                   .HasColumnType("varchar(50)")
                   .HasConversion(c=>PXPlusCryptor.Encrypt(c), c=>PXPlusCryptor.Decrypt(c));

            builder.OwnsOne(c => c.PayerInfo)
                   .Property(c => c.PayToolType)
                   .HasColumnName("PayToolType")
                   .HasColumnType("smallint");
            
            builder.OwnsOne(c => c.PayerInfo)
                   .Property(c => c.Sequence)
                   .HasColumnName("Sequence")
                   .HasColumnType("smallint");

            builder.OwnsOne(c => c.PayerInfo)
                   .Property(c => c.PhoneNumber)
                   .HasColumnName("PhoneNumber")
                   .HasColumnType("varchar(20)");

            builder.OwnsOne(c => c.PayerInfo)
                   .Property<string>("_bankCode")
                   .HasColumnName("BankCode")
                   .HasColumnType("varchar(3)");
            
            builder.OwnsOne(c => c.PayerInfo)
                   .Property<string>("_bankName")
                   .HasColumnName("BankName")
                   .HasColumnType("nvarchar(15)");

            builder.OwnsOne(c => c.ReceiveStoreInfo)
                   .Ignore(c => c.MerchantTradeNo);

            builder.OwnsOne(c => c.ReceiveStoreInfo)
                   .Property(c => c.MerchantCode)
                   .HasColumnName("MerchantCode")
                   .HasColumnType("varchar(50)");

            builder.OwnsOne(c => c.ReceiveStoreInfo)
                   .Property(c => c.StoreCode)
                   .HasColumnName("StoreCode")
                   .HasColumnType("varchar(50)");

            builder.OwnsOne(c => c.ReceiveStoreInfo)
                   .Property(c => c.StoreMemberId)
                   .HasColumnName("StoreMemberId")
                   .HasColumnType("bigint");

            builder.OwnsOne(c => c.ReceiveStoreInfo)
                   .Property(c => c.ReceiveAccount)
                   .HasColumnName("ReceiveAccount")
                   .HasColumnType("varchar(50)")
                   .HasConversion(c=>PXPlusCryptor.Encrypt(c), c=>PXPlusCryptor.Decrypt(c));

            builder.OwnsOne(c => c.ReceiveStoreInfo)
                   .Property(c => c.TradeFeeAmount)
                   .HasColumnName("TradeFeeAmount");

            builder.OwnsOne(c => c.ReceiveStoreInfo)
                   .Property(c => c.TradeFeeUnit)
                   .HasColumnName("TradeFeeUnit")
                   .HasColumnType("tinyint");

            builder.OwnsOne(c => c.ReceiveStoreInfo)
                   .Property(c => c.IndustryType)
                   .HasColumnName("IndustryType")
                   .HasColumnType("smallint");

            builder.OwnsOne(c => c.ReceiveStoreInfo)
                   .Property(c => c.BrandName)
                   .HasColumnName("BrandName")
                   .HasColumnType("nvarchar(100)");
            
            builder.OwnsOne(c => c.ReceiveStoreInfo)
                   .Property(c => c.MerchantName)
                   .HasColumnName("MerchantName")
                   .HasColumnType("nvarchar(100)");
            
            builder.OwnsOne(c => c.ReceiveStoreInfo)
                   .Property(c => c.StoreName)
                   .HasColumnName("StoreName")
                   .HasColumnType("nvarchar(100)");

            builder.OwnsOne(c => c.ReceiveStoreInfo)
                   .Ignore(c => c.Fees);
            
            builder.OwnsOne(c => c.PointDiscountInfo)
                   .Property(c => c.DiscountPercent)
                   .HasColumnName("DiscountPercent");

            builder.OwnsOne(c => c.PointDiscountInfo)
                   .Property(c => c.ActualDiscountAmount)
                   .HasColumnName("ActualDiscountAmount");

            builder.OwnsOne(c => c.PointDiscountInfo)
                   .Property(c => c.AvailableDiscountAmount)
                   .HasColumnName("AvailableDiscountAmount");
            
            builder.OwnsOne(c => c.PointDiscountInfo)
                   .Property(c => c.IsUsePoint)
                   .HasColumnName("IsUsePoint")
                   .HasColumnType("tinyint");

            builder.OwnsOne(c => c.BackInfo)
                   .Property(c => c.CashBack)
                   .HasColumnName("CashBack");
            
            builder.OwnsOne(c => c.BackInfo)
                   .Property(c => c.PointBack)
                   .HasColumnName("PointBack");

            builder.Ignore(c => c.ActivityFeedbackInfos);
            builder.Ignore(c => c.ActivityCashDiscountInfos);

            builder.Property<List<ActivityFeedbackInfo>>("_activityFeedbackInfos")
                   .HasColumnName("ActivityFeedbackInfos")
                   .HasColumnType("varchar(max)")
                   .HasConversion(
                       v => JsonSerializer.Serialize(v, null),
                       v => JsonSerializer.Deserialize<List<ActivityFeedbackInfo>>(v, null),
                       new ValueComparer<List<ActivityFeedbackInfo>>(
                           (c1, c2) => c1.SequenceEqual(c2),
                           c => c.Aggregate(0, (a, v) => HashCode.Combine(a, v.GetHashCode())),
                           c => c.ToList())
                   );

            builder.Property<List<ActivityCashDiscountInfo>>("_activityCashDiscountInfos")
                   .HasColumnName("ActivityCashDiscountInfos")
                   .HasColumnType("varchar(max)")
                   .HasConversion(
                       v => JsonSerializer.Serialize(v, null),
                       v => JsonSerializer.Deserialize<List<ActivityCashDiscountInfo>>(v, null),
                       new ValueComparer<List<ActivityCashDiscountInfo>>(
                           (c1, c2) => c1.SequenceEqual(c2),
                           c => c.Aggregate(0, (a, v) => HashCode.Combine(a, v.GetHashCode())),
                           c => c.ToList())
                   );

            builder.Property(c => c.PlatformType)
                   .HasColumnName("PlatformType")
                   .HasColumnType("tinyint");
            
            builder.Property(c=>c.CurrencyType)
                   .HasColumnName("CurrencyType")
                   .HasColumnType("smallint");
            
            builder.Ignore(c => c.InputMetaData);
            builder.Ignore(c => c.ReceiveMetaData);
            builder.Ignore(c => c.BankReturnCode);
            builder.Ignore(c => c.RefundOrderNo);
            builder.Ignore(c => c.Executor);
            builder.Ignore(c => c.RefundCause);
            builder.Ignore(c => c.ExecuteSource);
        }
    }
}