﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Text;
using AccountForPaymentService;
using AccountLinkService;
using CreditCardPaymentService;
using FluentValidation;
using IdGen;
using MediatR;
using MerchantForPaymentService;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using PointGrpcService;
using PXPlus.Core.EventBusKafka.Extensions.DependencyInjection;
using PXPlus.Core.GrpcInterceptor;
using PXPlus.Core.IntegrationEventLogEf;
using PXPlus.Core.IntegrationEventLogEf.Extension.DependencyInjection;
using PXPlus.Core.NLog.SensitiveDataMask;
using PXPlus.Core.RedisCache.Register;
using PXPlus.Core.Security;
using PXPlus.Payment.Application.Behaviors;
using PXPlus.Payment.Application.Validations;
using PXPlus.Payment.Domain.Events;
using PXPlus.Payment.Domain.OrderAggregate;
using PXPlus.Payment.Domain.OrderRecordAggregate;
using PXPlus.Payment.Domain.SeedWork;
using PXPlus.Payment.Infrastructure;
using PXPlus.Payment.Infrastructure.DomainServiceGrpc;
using PXPlus.Payment.Infrastructure.Idempotency;
using PXPlus.Payment.Infrastructure.Repositories;
using PXPlus.Payment.Api.Infrastructure.Interceptors;
using PXPlus.Payment.Api.Infrastructure.Mapper;
using PXPlus.Payment.Api.Services;
using PXPlus.Payment.Application.Configs;
using PXPlus.Payment.Application.DomainEventHandlers.OrderPayment;
using PXPlus.Payment.Application.Queries;
using PXPlus.Payment.Domain.DomainServices.Interface;
using PXPlus.Payment.Domain.Factories.Implement;
using PXPlus.Payment.Domain.Factories.Interface;
using PXPlus.Payment.Infrastructure.Configs;
using PXPlus.Payment.Infrastructure.DomainServiceGrpc.ExecutePaymentDomainServices;
using PXPlus.Payment.Infrastructure.DomainServiceImp;
using PXPlus.Payment.Infrastructure.Factories;
using RefundGrpcService = PXPlus.Payment.Api.Services.RefundGrpcService;


namespace PXPlus.Payment.Api
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        // For more information on how to configure your application, visit https://go.microsoft.com/fwlink/?LinkID=398940
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddGrpc(c =>
                    {
                        c.Interceptors.Add<GrpcGlobalExceptionHandleInterceptor>();
                        c.Interceptors.Add<GrpcGlobalRequestIdInterceptor>();
                    })
                    .Services
                    .AddCustomPipelineBehavior()
                    .AddDomainService()
                    .AddRepository()
                    .AddFactories()
                    .AddCustomDbContext(Configuration)
                    .AddCustomGrpcClient(Configuration)
                    .AddEventBus(Configuration)
                    .AddRedisCache(Configuration);

            services.AddScoped<IRequestManager, RequestManager>(c =>
            {
                var context = c.GetService<PaymentContext>();
                return new RequestManager(context);
            });

            services.AddSingleton<IPaymentQuotaDomainService, PaymentQuotaGrpcDomainService>();
            services.AddSingleton<IPayToolDomainService, PayToolGrpcDomainService>();
            services.AddSingleton<IExecutePaymentDomainServiceFactory, ExecutePaymentDomainServiceFactory>();
            
            
            services.AddSingleton<IExecutePaymentDomainService, ExecuteCreditCardPaymentGrpcDomainService>();
            services.AddSingleton<IExecutePaymentDomainService, ExecuteAcLinkPaymentGrpcDomainService>();
            services.AddSingleton<IExecutePaymentDomainService, ExecuteEpAccountPaymentGrpcDomainService>();
            
            
            #region Configs

            services.Configure<PaymentQuotaLimitConfig>(Configuration.GetSection("PaymentQuotaLimitConfig"));
            services.Configure<OrderNoPrefixConfig>(Configuration);
            #endregion

            services.AddSingleton<PaymentAppResMapToGrpcRes>();
            services.AddSingleton<RefundAppResMapToGrpcRes>();

            services.AddScoped<QueryAppService>();
         
            services.AddScoped<IUnitOfWork, PaymentContext>();

            services.AddMediatR(typeof(Startup).Assembly
                , typeof(OrderPaymentSuccessfulDomainEvent).Assembly
                , typeof(OrderPaymentSuccessfulDomainEventHandler).Assembly);

            AssemblyScanner.FindValidatorsInAssembly(typeof(PaymentCommandValidator).Assembly)
                           .ForEach(item => services.AddScoped(item.InterfaceType, item.ValidatorType));

            services.AddAutoMapper(typeof(Startup));

            //todo 
            PXPlusCryptor.SetInit("pxPAyPlUsAccOUnt2021011416527878", "@aCCounTPxPlus34");
            
            services.AddSingleton(_=>new IdGenerator(0));

            services.MaskSensitiveData();
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseRouting();

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapGrpcService<PaymentGrpcService>();
                endpoints.MapGrpcService<RefundGrpcService>();
                endpoints.MapGrpcService<QueryGrpcService>();
                endpoints.MapGrpcService<CancelPaymentGrpcService>();
                endpoints.MapGrpcService<ConfirmOrderGrpcService>();

                endpoints.MapGet("/",
                    async context =>
                    {
                        await context.Response.WriteAsync(
                            "Communication with gRPC endpoints must be made through a gRPC client. To learn how to create a client, visit: https://go.microsoft.com/fwlink/?linkid=2086909");
                    });
            });
        }

    }

    static class CustomExtensionsMethods
    {
        public static IServiceCollection AddDomainService(this IServiceCollection services)
            => services.AddSingleton<IPointDomainService, PointGrpcDomainService>()
                       .AddSingleton<IMerchantDomainService, MerchantGrpcDomainService>()
                       .AddSingleton<IActivityDomainService, ActivityMockDomainService>()
                       .AddSingleton<IGenerateOrderNoDomainService,GenerateOrderNoDomainService>();

        public static IServiceCollection AddRepository(this IServiceCollection services)
            => services.AddScoped<IOrderRepository, OrderRepository>()
                       .AddScoped<IOrderRecordRepository, OrderRecordRepository>();

        public static IServiceCollection AddFactories(this IServiceCollection services)
            => services.AddSingleton<IOrderFactory, OrderFactory>();
                      
        public static IServiceCollection AddCustomDbContext(this IServiceCollection services,
            IConfiguration configuration)
        {
            services.AddDbContext<PaymentContext>(options =>
                {
                    options.UseSqlServer(configuration["ConnectionString"],
                        sqlOptions =>
                        {
                            sqlOptions.MigrationsAssembly(typeof(Startup).GetTypeInfo().Assembly.GetName().Name);
                        });
                }
                , ServiceLifetime.Scoped
            );

            services.AddDbContext<IntegrationEventLogContext>(options =>
            {
                options.UseSqlServer(configuration["ConnectionString"],
                    sqlOptions =>
                    {
                        sqlOptions.MigrationsAssembly(typeof(Startup).GetTypeInfo().Assembly.GetName().Name);
                    });
            });


            return services;
        }

        public static IServiceCollection AddCustomGrpcClient(this IServiceCollection services, IConfiguration configuration)
        {
            services.AddGrpcLoggingInterceptor();
            services.AddGlobalRequestIdInterceptor();
            
            var accountLink = configuration.GetSection("AccountLink:Endpoint").Value;
            
            services.AddGrpcClient<AccountLinkGrpcService.AccountLinkGrpcServiceClient>(c=>
                    c.Address = new Uri(accountLink))
                .AddInterceptor<GrpcGlobalRequestIdInterceptor>()
                .AddInterceptor<GrpcLoggingInterceptor>();

            var accountService = configuration.GetSection("AccountService:Endpoint").Value;
            
            services.AddGrpcClient<CreditCardPayToolForPaymentGrpcService.CreditCardPayToolForPaymentGrpcServiceClient>(
                    c => c.Address = new Uri(accountService))
                .AddInterceptor<GrpcGlobalRequestIdInterceptor>()
                .AddInterceptor<GrpcLoggingInterceptor>();

            services.AddGrpcClient<EpAccountPayToolForPaymentGrpcService.EpAccountPayToolForPaymentGrpcServiceClient>(
                    c => c.Address = new Uri(accountService))
                .AddInterceptor<GrpcGlobalRequestIdInterceptor>()
                .AddInterceptor<GrpcLoggingInterceptor>();

            services.AddGrpcClient<EpAccountForPaymentGrpcService.EpAccountForPaymentGrpcServiceClient>(
                    c => c.Address = new Uri(accountService))
                .AddInterceptor<GrpcGlobalRequestIdInterceptor>()
                .AddInterceptor<GrpcLoggingInterceptor>();
            
                      
            services.AddGrpcClient<AccountForPaymentGrpcService.AccountForPaymentGrpcServiceClient>(
                    c => c.Address = new Uri(accountService))
                .AddInterceptor<GrpcGlobalRequestIdInterceptor>()
                .AddInterceptor<GrpcLoggingInterceptor>();
            
            services.AddGrpcClient<AccountLinkPayToolForPaymentGrpcService.AccountLinkPayToolForPaymentGrpcServiceClient>(
                    c => c.Address = new Uri(accountService))
                .AddInterceptor<GrpcGlobalRequestIdInterceptor>()
                .AddInterceptor<GrpcLoggingInterceptor>();
            
            var creditcard = configuration.GetSection("CreditCard:Endpoint").Value;
            
            services.AddGrpcClient<PayGrpcService.PayGrpcServiceClient>(
                    c => c.Address = new Uri(creditcard))
                .AddInterceptor<GrpcGlobalRequestIdInterceptor>()
                .AddInterceptor<GrpcLoggingInterceptor>();


            var merchant = configuration.GetSection("Merchant:Endpoint").Value;
            
            services.AddGrpcClient<MerchantForPaymentGrpcService.MerchantForPaymentGrpcServiceClient>(
                c=>c.Address = new Uri(merchant))
                .AddInterceptor<GrpcGlobalRequestIdInterceptor>()
                .AddInterceptor<GrpcLoggingInterceptor>();

            var point = configuration.GetSection("PointService:Endpoint").Value;
            
            services.AddGrpcClient<OperatePointGrpcService.OperatePointGrpcServiceClient>(
                c=>c.Address = new Uri(point))
                .AddInterceptor<GrpcGlobalRequestIdInterceptor>()
                .AddInterceptor<GrpcLoggingInterceptor>();
            
            return services;
        }

        public static IServiceCollection AddCustomPipelineBehavior(this IServiceCollection services)
            => services.AddTransient(typeof(IPipelineBehavior<,>), typeof(LoggingBehavior<,>))
                        .AddTransient(typeof(IPipelineBehavior<,>), typeof(ExceptionHandleBehavior<,>))
                        .AddTransient(typeof(IPipelineBehavior<,>), typeof(ValidatorBehavior<,>))
                        .AddTransient(typeof(IPipelineBehavior<,>), typeof(TransactionBehavior<,>));

        public static IServiceCollection AddEventBus(this IServiceCollection services,IConfiguration configuration)
        {
            services.RegisterIntegrationEventSource<PaymentContext>();
            
            services.RegisterEventBusKafka(c =>
            {
                c.ProducerOptions = new ProducerOptions
                {
                    EndPoints = configuration["EventBus:Endpoints"],
                    RetryCount = 3
                };
            });
            
            services.RegisterIntegrationEventDelayPublish();
            
            return services;
        }

        public static IServiceCollection AddRedisCache(this IServiceCollection services, IConfiguration configuration)
        {
            services.RegisterRedisLink(configuration["RedisCache:Endpoints"]);
            return services;
        }

        public static IServiceCollection MaskSensitiveData(this IServiceCollection services)
        {
            services.RegisterNLogSensitiveDataMask(new NLogSensitiveDataMaskOptions
            {
                MaskDataDic = new Dictionary<string, Func<string, string>>
                {
                    {"PhoneNumber", c =>
                    {
                        if (!string.IsNullOrEmpty(c) && c.Length >= 10)
                            return c.Remove(2, 5).Insert(2, "*****");
                        return c;
                    }},
                    {"identityNo",MaskIdentityNo},
                    {"virtualAccount", MaskAccountLink},
                    {"IdentityNo",MaskIdentityNo},
                    {"VirtualAccount",MaskAccountLink},
                    {"PayerPaymentAccount",MaskAccountLink},
                    {"creditCard",MaskCreditCard},
                    {"CreditCard",MaskCreditCard}
                }
            });
            
            string MaskAccountLink(string virtualAccount)
            {
                if (!string.IsNullOrEmpty(virtualAccount) && virtualAccount.Length > 6)
                {
                    var startIndex = virtualAccount.Length - 5;
                    return "*" + virtualAccount[startIndex..];
                }

                return "*" + virtualAccount;
            }

            string MaskCreditCard(string creditCard)
                => creditCard.Remove(6, 6).Insert(6,"******");

            string MaskIdentityNo(string identityNo)
            {
                if (! string.IsNullOrEmpty(identityNo) && identityNo.Length >= 10)
                    return identityNo[..3] + "*****" + identityNo.Substring(7, 3);
                return identityNo;
            }


            return services;
        }
    }
}