﻿using System.Collections.Generic;
using System.Linq;
using System.Text.Json;
using System.Threading.Tasks;
using CorrelationId;
using CorrelationId.Abstractions;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using PaymentService;
using PXPlus.Cache.Abstract;
using PXPlus.Core.Extensions;
using PXPlus.Payment.Application.Commands;
using PXPlus.Payment.Domain.DomainExceptions;
using PXPlus.PaymentCancel.Worker.Payment;
using Quartz;
using Activity = PaymentService.Activity;

namespace PXPlus.PaymentCancel.Worker.Jobs
{
    [DisallowConcurrentExecution]
    public class CancelPaymentJob : IJob
    {
        private readonly ILogger<CancelPaymentJob> _logger;
        private readonly ICacheLinkList _cacheLinkList;
        private readonly IRefundService _refundService;
        private readonly IQueryService _queryService;
        private readonly Config _config;
        private readonly ICorrelationContextAccessor _correlationContextAccessor;

        public CancelPaymentJob(
            ILogger<CancelPaymentJob> logger,
            ICacheLinkList cacheLinkList, 
            IRefundService refundService,
            IQueryService queryService, 
            IOptionsMonitor<Config> config,
            ICorrelationContextAccessor correlationContextAccessor)
        {
            _logger = logger;
            _cacheLinkList = cacheLinkList;
            _refundService = refundService;
            _queryService = queryService;
            _config = config.CurrentValue;
            _correlationContextAccessor = correlationContextAccessor;
        }

        public async Task Execute(IJobExecutionContext context)
        {
            var count = await _cacheLinkList.LLengthAsync(_config.PaymentCancelRedisKey);
            
            _logger.LogInformation("Beginning job ... pending refund count {@Count}",count);
            
            for (var i = 0; i < count; i++)
            {
                var reversDataStr = await _cacheLinkList.LPopAsync(_config.PaymentCancelRedisKey);

                var reversData = JsonSerializer.Deserialize<CancelPaymentCommand>(reversDataStr);

                if (reversData == null)
                    continue;

                _correlationContextAccessor.CorrelationContext = new CorrelationContext("x-request-id",reversData.XRequestId);
                
                using (_logger.BeginScope(new[] {new KeyValuePair<string, object>("requestid", reversData.XRequestId)}))
                {
                    if (reversData.RetryCount > _config.RefundRetryTime)
                    {
                        _logger.LogInformation("Order retry count > {@RetryTime} times : {@ReversData}",_config.RefundRetryTime, reversData);
                        await _cacheLinkList.LPushAsync(_config.PaymentCancelDeadStillRedisKey,
                            JsonSerializer.Serialize(reversData));
                        continue;
                    }

                    var queryGrpcResponse = await _queryService.QueryAsync(new QueryGrpcRequest
                    {
                        TradeNo = reversData.TradeNo
                    });

                    if (queryGrpcResponse.Code != "0000" || queryGrpcResponse.Data.PaymentInfo.Code ==
                        PaymentStatusCode.TradeProcessing.GetResultCode())
                    {
                        //retry 
                        reversData.RetryCount++;
                        await _cacheLinkList.LPushAsync(_config.PaymentCancelRedisKey, JsonSerializer.Serialize(reversData));
                        continue;
                    }

                    if (queryGrpcResponse.Data.OrderType != QueryOrderType.PaymentOrder)
                    {
                        // 非付款單 不retry
                        _logger.LogError("Order is not payment order ! : {@ReversData}", reversData);
                        continue;
                    }

                    if (queryGrpcResponse.Data.PaymentInfo.Code == "0000")
                    {
                        var paymentResult = queryGrpcResponse.Data.PaymentInfo.Data;

                        var refundRequest =
                            new RefundGrpcRequest
                            {
                                TradeNo = reversData.TradeNo + reversData.RetryCount,
                                OriPxTradeNo = paymentResult.PxTradeNo,
                                RefunderId = reversData.PayerId,
                                Amount = paymentResult.OrderAmount,
                                NoneDiscountAmount = paymentResult.NoneDiscountAmount,
                                NoneFeedbackAmount = paymentResult.NoneFeedbackAmount,
                                StoreCode = reversData.StoreCode,
                                Cause = "付款取消 - 沖銷",
                                Executor = "Cancel Payment Job",
                                ExecuteSource = 2
                            };
                        
                        if (paymentResult.Activities.Any())
                        {
                            refundRequest.Activities.AddRange(paymentResult.Activities.Select(c => new Activity
                            {
                                ActivityCode = c.ActivityCode,
                                ProductAmount = c.ProductAmount
                            }));
                        }
                        
                        //refund
                        var result = await _refundService.RefundCommandAsync(refundRequest);

                        if (result.Code == "0000")
                            _logger.LogInformation("Order {@ReversData} refund successful", reversData);
                        else
                            _logger.LogInformation("Order {@ReversData} refund failed", reversData);
                    }
                    else
                    {
                        _logger.LogInformation("Order {@ReversData} payment failed", reversData);
                    }
                }
            }

            _logger.LogInformation("Completed job ...");
        }
    }
}