using System;
using System.Collections.Generic;
using System.Reflection;
using CorrelationId.DependencyInjection;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using PaymentService;
using PXPlus.Core.EventBusKafka.Extensions.DependencyInjection;
using PXPlus.Core.GrpcInterceptor;
using PXPlus.Core.IntegrationEventLogEf.Extension.DependencyInjection;
using PXPlus.Core.RedisCache.Register;
using PXPlus.Payment.Application.IntegrationEvents.Events;
using PXPlus.PaymentCancel.Worker.Interceptors;
using PXPlus.PaymentCancel.Worker.Jobs;
using PXPlus.PaymentCancel.Worker.Payment;
using Quartz;

namespace PXPlus.PaymentCancel.Worker
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddControllers();
            
            services.AddRedisCache(Configuration)
                    .AddPaymentServiceGrpc(Configuration)
                    .AddGrpcLoggingInterceptor()
                    .AddGlobalRequestIdInterceptor();
       
            
            services.AddQuartz(q =>
            {
                q.UseMicrosoftDependencyInjectionJobFactory(options =>
                {
                    options.CreateScope = false;
                });

                q.AddJob<CancelPaymentJob>(c => c.StoreDurably());
        
                // also add XML configuration and poll it for changes
                q.UseXmlSchedulingConfiguration(x =>
                {
                    x.Files = new[] { "~/quartz_jobs.config" };
               });
            });

            services.AddTransient<CancelPaymentJob>();
            services.Configure<Config>(Configuration.GetSection("Config"));
            services.AddCorrelationId();
            services.AddSingleton<WorkerRequestIdInterceptor>();
            services.AddQuartzHostedService(options =>
            {
                // when shutting down we want jobs to complete gracefully
                options.WaitForJobsToComplete = true;
            });

            services.AddEventBusChecker(Configuration);
            
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseRouting();

            app.UseEndpoints(endpoints => { endpoints.MapControllers(); });
        }
    }
    
    static class CustomExtensionsMethods
    {
        public static IServiceCollection AddRedisCache(this IServiceCollection services, IConfiguration configuration)
        {
            services.RegisterRedisLink(configuration["RedisCache:Endpoints"]);
            return services;
        }

        public static IServiceCollection AddPaymentServiceGrpc(this IServiceCollection services,
            IConfiguration configuration)
        {
            
            services.AddSingleton<IRefundService, RefundService>();
            services.AddSingleton<IQueryService, QueryService>();
            
            var paymentEndpoint = configuration.GetSection("PaymentService:Endpoint").Value;
            
            services.AddGrpcClient<RefundGrpcService.RefundGrpcServiceClient>(
                c=>c.Address = new Uri(paymentEndpoint))
                .AddInterceptor<WorkerRequestIdInterceptor>()
                .AddInterceptor<GrpcLoggingInterceptor>();
        
            services.AddGrpcClient<QueryGrpcService.QueryGrpcServiceClient>(
                c=>c.Address = new Uri(paymentEndpoint))
                .AddInterceptor<WorkerRequestIdInterceptor>()
                .AddInterceptor<GrpcLoggingInterceptor>();

            return services;
        }

        public static IServiceCollection AddEventBusChecker(this IServiceCollection services,
            IConfiguration configuration)
        {
            var eventBusEndPoint = configuration["EventBusKafka"];

            services.RegisterEventBusKafka(c =>
            {
                c.ProducerOptions = new ProducerOptions
                {
                    EndPoints = eventBusEndPoint,
                    RetryCount = 5
                };
            });
            
            var sqlConnectionStr = configuration["ConnectionString"];
            
            services.RegisterIntegrationEventPublishedFailedChecker(sqlConnectionStr
                ,new List<Assembly>
                {
                    typeof(OrderPaymentSuccessfulIntegrationEvent).Assembly
                });
            return services;
        }
    }
    
    
}